@@ -3,8 +3,16 @@
 // #ifndef USE_RINTERNALS
 // #define USE_RINTERNALS
 // #endif
+#include "base_radixsort.h"
 #include <math.h>
 
+// // Function to check if two strings have the same encoding
+// int are_encodings_equal(SEXP str1, SEXP str2) {
+//   cetype_t enc1 = Rf_getCharCE(STRING_ELT(str1, 0));
+//   cetype_t enc2 = Rf_getCharCE(STRING_ELT(str2, 0));
+//   return enc1 == enc2;
+// }
+
 void matCopyAttr(SEXP out, SEXP x, SEXP Rdrop, int ng) {
   SEXP dn = getAttrib(x, R_DimNamesSymbol);
   SEXP cn = isNull(dn) ? R_NilValue : VECTOR_ELT(dn, 1); // PROTECT ??
@@ -65,7 +73,7 @@ SEXP falloc(SEXP value, SEXP n, SEXP simplify)  {
       break;
     }
     case STRSXP: {
-      SEXP val = asChar(value), *pout = STRING_PTR(out);
+      SEXP val = asChar(value), *pout = SEXPPTR(out);
       for(int i = 0; i != l; ++i) pout[i] = val;
       break;
     }
@@ -168,7 +176,7 @@ SEXP multiassign(SEXP lhs, SEXP rhs, SEXP envir) {
     return R_NilValue;
   }
   if(length(rhs) != n) error("length(lhs) must be equal to length(rhs)");
-  SEXP *plhs = STRING_PTR(lhs);
+  SEXP *plhs = SEXPPTR(lhs);
   switch(TYPEOF(rhs)) { // installTrChar translates to native encoding, installChar does the same now, but also is available on older systems.
     case REALSXP: {
       double *prhs = REAL(rhs);
@@ -187,7 +195,7 @@ SEXP multiassign(SEXP lhs, SEXP rhs, SEXP envir) {
       break;
     }
     case STRSXP: {
-      SEXP *prhs = STRING_PTR(rhs);
+      SEXP *prhs = SEXPPTR(rhs);
       for(int i = 0; i < n; ++i) {
         SEXP nam = installChar(plhs[i]);
         defineVar(nam, ScalarString(prhs[i]), envir);
@@ -234,11 +242,17 @@ SEXP vlabels(SEXP x, SEXP attrn, SEXP usenam) {
     return labx;
   }
   SEXP res = PROTECT(allocVector(STRSXP, l));
-  SEXP *pres = STRING_PTR(res);
+  SEXP *pres = SEXPPTR(res);
   const SEXP *px = SEXPPTR_RO(x);
   for(int i = 0; i < l; ++i) {
     SEXP labxi = getAttrib(px[i], sym_attrn);
-    pres[i] = TYPEOF(labxi) == STRSXP ? STRING_ELT(labxi, 0) : labxi == R_NilValue ? NA_STRING : asChar(labxi);
+    if(TYPEOF(labxi) == STRSXP) pres[i] = STRING_ELT(labxi, 0);
+    else if(labxi == R_NilValue) pres[i] = NA_STRING;
+    else {
+      PROTECT(labxi);
+      pres[i] = asChar(labxi);
+      UNPROTECT(1);
+    }
   }
   if(asLogical(usenam)) {
     SEXP nam = getAttrib(x, R_NamesSymbol);
@@ -249,8 +263,6 @@ SEXP vlabels(SEXP x, SEXP attrn, SEXP usenam) {
 }
 
 // Note: ind can be NULL...
-// TODO: option to shallow copy attributes list for setLabels / relabel !!!!
-
 SEXP setvlabels(SEXP x, SEXP attrn, SEXP value, SEXP ind) { // , SEXP sc
  if(!isString(attrn)) error("'attrn' must be of mode character");
  if(length(attrn) != 1) error("exactly one attribute 'attrn' must be given");
@@ -560,32 +572,22 @@ SEXP fnrowC(SEXP x) {
   return ScalarInteger(INTEGER(dim)[0]);
 }
 
+
+#define MYEFL(x) (((SEXPREC_partial *)(x))->sxpinfo.gp)
+#define MYSEFL(x,v)	((((SEXPREC_partial *)(x))->sxpinfo.gp)=(v))
+
 // Taken from: https://github.com/r-lib/rlang/blob/main/src/internal/env.c
 #define CLP_FRAME_LOCK_MASK (1 << 14)
-#define CLP_FRAME_IS_LOCKED(e) (ENVFLAGS(e) & CLP_FRAME_LOCK_MASK)
-#define CLP_UNLOCK_FRAME(e) SET_ENVFLAGS(e, ENVFLAGS(e) & (~CLP_FRAME_LOCK_MASK))
+#define CLP_FRAME_IS_LOCKED(e) (MYEFL(e) & CLP_FRAME_LOCK_MASK)
+#define CLP_UNLOCK_FRAME(e) MYSEFL(e, MYEFL(e) & (~CLP_FRAME_LOCK_MASK))
 
-SEXP unlock_collapse_namespace(SEXP env, SEXP macros) {
+SEXP unlock_collapse_namespace(SEXP env) {
   if(TYPEOF(env) != ENVSXP) error("Unsupported object passed to C_unlock_collapse_namespace: %s", type2char(TYPEOF(env)));
   CLP_UNLOCK_FRAME(env);
-  if(asLogical(macros)) {
-    R_unLockBinding(install(".FAST_STAT_FUN_EXT"), env);
-    R_unLockBinding(install(".FAST_STAT_FUN_POLD"), env);
-    R_unLockBinding(install(".FAST_FUN_MOPS"), env);
-    R_unLockBinding(install(".COLLAPSE_ALL_EXPORTS"), env);
-  }
+  R_unLockBinding(install(".FAST_STAT_FUN_EXT"), env);
+  R_unLockBinding(install(".FAST_STAT_FUN_POLD"), env);
+  R_unLockBinding(install(".FAST_FUN_MOPS"), env);
+  R_unLockBinding(install(".COLLAPSE_ALL_EXPORTS"), env);
   return CLP_FRAME_IS_LOCKED(env) == 0 ? ScalarLogical(1) : ScalarLogical(0);
 }
 
-
-// SEXP dot_return(SEXP x) {
-//   SEXP nam = getAttrib(x, R_NamesSymbol);
-//   if(TYPEOF(nam) == STRSXP) {
-//     int l = length(nam);
-//     SEXP *pnam = STRING_PTR(nam);
-//     for (int i = 0; i < l; ++i) if(length(pnam[i]) == 0) pnam[i] = 0;
-//
-//     substitute(x, R_NilValue);
-//
-//   }
-// }
